#include <Arduino.h>
#include <micro_ros_platformio.h>

#include "pins.hpp"

#include <rcl/rcl.h>
#include <rclc/rclc.h>
#include <rclc/executor.h>

#include <std_msgs/msg/int32.h>

#if !defined(MICRO_ROS_TRANSPORT_ARDUINO_SERIAL)
#error This example is only avaliable for Arduino framework with serial transport.
#endif

//rcl_subscription_t subscriber;


rcl_publisher_t button_left_publisher;
rcl_publisher_t button_middle_publisher;
rcl_publisher_t button_right_publisher;
rcl_publisher_t tirette_publisher;

std_msgs__msg__Int32 msg;

rclc_executor_t executor;
rclc_support_t support;
rcl_allocator_t allocator;
rcl_node_t node;
rcl_timer_t timer;

#define RCCHECK(fn) { rcl_ret_t temp_rc = fn; if((temp_rc != RCL_RET_OK)){error_loop();}}
#define RCSOFTCHECK(fn) { rcl_ret_t temp_rc = fn; if((temp_rc != RCL_RET_OK)){}}

// Error handle loop
void error_loop() {
  while(1) {
    delay(100);
  }
}

int mult = 0;
void timer_callback(rcl_timer_t * timer, int64_t last_call_time) {
  msg.data = !digitalRead(BUTTON_LEFT);
  RCSOFTCHECK(rcl_publish(&button_left_publisher, &msg, NULL));
  msg.data = !digitalRead(BUTTON_MIDDLE);
  RCSOFTCHECK(rcl_publish(&button_middle_publisher, &msg, NULL));
  msg.data = !digitalRead(BUTTON_RIGHT);
  RCSOFTCHECK(rcl_publish(&button_right_publisher, &msg, NULL));
  msg.data = !digitalRead(TIRETTE);
  RCSOFTCHECK(rcl_publish(&tirette_publisher, &msg, NULL));
}

void setup() {
  // Configure serial transport
  Serial.begin(115200);
  set_microros_serial_transports(Serial);
  delay(2000);

  pinMode(BUTTON_LEFT, INPUT_PULLUP);
  pinMode(BUTTON_MIDDLE, INPUT_PULLUP);
  pinMode(BUTTON_RIGHT, INPUT_PULLUP);
  pinMode(TIRETTE, INPUT_PULLUP);

  allocator = rcl_get_default_allocator();

  //create init_options
  RCCHECK(rclc_support_init(&support, 0, NULL, &allocator));

  // create node
  RCCHECK(rclc_node_init_default(&node, "ctrl_panel_ucontroller", "", &support));



  RCCHECK(rclc_publisher_init_default(
    &button_left_publisher,
    &node,
    ROSIDL_GET_MSG_TYPE_SUPPORT(std_msgs, msg, Int32),
    "ctrl_panel/left_button"));

  RCCHECK(rclc_publisher_init_default(
    &button_middle_publisher,
    &node,
    ROSIDL_GET_MSG_TYPE_SUPPORT(std_msgs, msg, Int32),
    "ctrl_panel/middle_button"));

  RCCHECK(rclc_publisher_init_default(
    &button_right_publisher,
    &node,
    ROSIDL_GET_MSG_TYPE_SUPPORT(std_msgs, msg, Int32),
    "ctrl_panel/right_button"));

  RCCHECK(rclc_publisher_init_default(
    &tirette_publisher,
    &node,
    ROSIDL_GET_MSG_TYPE_SUPPORT(std_msgs, msg, Int32),
    "ctrl_panel/tirette"));

  const unsigned int timer_timeout = 10;
  RCCHECK(rclc_timer_init_default(
    &timer,
    &support,
    RCL_MS_TO_NS(timer_timeout),
    timer_callback));

  // create executor
  RCCHECK(rclc_executor_init(&executor, &support.context, 2, &allocator));
  RCCHECK(rclc_executor_add_timer(&executor, &timer));
  //RCCHECK(rclc_executor_add_subscription(&executor, &subscriber, &msg, &subscription_callback, ON_NEW_DATA));
  msg.data = 0;
}

void loop() {
  RCSOFTCHECK(rclc_executor_spin_some(&executor, RCL_US_TO_NS(1000)));
}